/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */



#ifndef UMASS_UMASSENUM_H_INCLUDED
#define UMASS_UMASSENUM_H_INCLUDED




// Module Description:  header for umass-enum driver

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/queue.h>
#include <fcntl.h>
#include <spawn.h>
#include <signal.h>
#include <stdarg.h>
#include <pthread.h>
#include <sys/mount.h>
#include <sys/neutrino.h>
#include <sys/procmgr.h>
#include <sys/slog.h>
#include <sys/usbdi.h>

#ifdef UMASS_INCLUDE_PRIORITY_BOOST
#include <hw/dcmd_sim_umass.h>
#endif


#include "resmgr-enum.h"
/* 
               Driver to manage USB mass storage devices
*/
#define DEFAULT_MOUNT_LOCATION 				"/dev/umass"
#define DEFAULT_DEVICE_PREFIX 				"hdumass"
#define DEFAULT_DEVICE_CDPREFIX 			"cdumass"
#define DEFAULT_MOUNT_ENTRY 				"/fs/hdumass"
#define DEFAULT_CDMOUNT_ENTRY 				"/fs/cdumass"

#define UMASS_DEFAULT_PRIORITY				21
#define UMASS_DEFAULT_UNKNOWN_PRIORITY		UMASS_DEFAULT_PRIORITY
#define UMASS_DEFAULT_RMB_MEDIA_POLL		1000
#define DEFAULT_CONFIG_FILE 				"/etc/config/umass-enum.cfg"
#define PARSER_MAX_LINE_LENGTH				1024
#define DEFAULT_MAX_LUN						8
#define UMASS_AUTORUN_SCRIPT				"autorun"

#define UMASS_IDENT_SUBCLASS_CDROM 			0x02

// overide default cache allocation, we control mounts,
// don't delay writes, register device entries under /umass
#define UMASS_MGR_DEFAULT_BLK_OPTIONS 		"cache=50k,auto=none,devdir=" // we will fill in devdir
#define UMASS_MGR_DEFAULT_CAM_OPTIONS 		"quiet"

#define MSEC 1000000L

// define some slog codes
#define _SLOGC_UAMASS_ENUM_GEN 			0x8000
#define _SLOG_UMASS_ERROR				0x0001

typedef struct _umass_mount_list {
	TAILQ_ENTRY( _umass_mount_list ) 	mlink;
	char								*partition;
	char								*mount_point;
	char								*fstype;
	int									lun;
	int									mount_flags;
} umass_mount_list;

typedef struct _umass_mounted_list {
	TAILQ_ENTRY( _umass_mounted_list ) 	mlink;
	int									lun;
	char								fstype[5];
	_uint32								mount_flags;
	char								partition[4];
	char								mount_point[50];
	char								mount_as[50];
} umass_mounted_list;


typedef struct _mass_device_info {
	TAILQ_ENTRY( _mass_device_info ) 	dlink;
	_uint32								flags;
	char								*args_buff;
	_uint32								vendor;
	_uint32								device;
	_uint32								dclass;
	_uint32								subclass;
	_uint32								protocol;
	char 								*serial_num;
	char 								*blk;
	char 								*cam;
	char 								*disk;
	char 								*cdrom;
	char 								*umass;
	char								*qnx4;
	char 								*dos;
	char 								*cd;
	char								*scsi_mask;
	_uint32								default_access;
	TAILQ_HEAD(,_umass_mount_list ) 	mount_list;
	char								*comment;
	int									timeout;
} mass_device_info;

#define MASS_FLAGS_ALLOW_WRITE			0x01


typedef struct _umass_lun {
	_uint32		lun_change_count;
	_uint32		flags;
} umass_lun_t;

#define UMASS_LUN_MEDIA_INSERTED	0x01
#define UMASS_LUN_MEDIA_REMOVED		0x02
#define UMASS_LUN_MEDIA_REMOVEABLE	0x04

typedef struct _current_device_info {
	TAILQ_ENTRY( _current_device_info ) 	dlink;
	_uint32									flags;
	pid_t									pid;
	int										dev_num;
	mass_device_info						*mentry;
	int										num_luns;
	umass_lun_t								luns[DEFAULT_MAX_LUN];
	TAILQ_HEAD(,_umass_mounted_list ) 		current_mounts;
	struct usbd_device_instance				instance;
	_uint32     							upstream_port;
	_uint32     							upstream_devno;
	_uint32     							upstream_port_speed;
	char									*vendor_string;
	char									*device_string;
	char									*serial_string;
} current_device_info_t;



#define UMASS_CFLAGS_DEVICE_NOT_USED	0x00000001
#define UMASS_CFLAGS_ALLOW_WRITE		0x00000002
#define UMASS_CFLAGS_DRIVER_DIED  		0x80000000

typedef struct _mass_mgr_ctrl {
	TAILQ_HEAD(,_mass_device_info )    dlist; // linked list of entries from file
	TAILQ_HEAD(,_current_device_info ) cdlist; // linked list of entries from file
	_uint32								cflags;
	_uint32								verbose;
	_uint32								umass_priority;	
	_uint32								umass_unknown_priority;	
	timer_t                 			timer;
	_uint32								rmb_poll;
	_uint8								*prefix;
	_uint8								*location;
	_uint8								*mount_prefix;
	_uint8								*cdmount_prefix;
	_uint8								*mount_name;
	_uint8								*cdmount_name;
	_uint32								flags;
	_uint32								default_access;
	_uint32								max_device_lun;
	struct usbd_connection				*connection;
	struct _umass_resmgr_ctrl			*rctrl;			// used for resmgr interface
} mass_mgr_ctrl_t;

mass_mgr_ctrl_t MassCtrl;

#define UMASS_FLAGS_NO_AUTOMOUNT 		0x01
#define UMASS_FLAGS_TEST_UNKOWN_DRIVES	0x02
#define UMASS_FLAGS_ADD_UNKOWN_DRIVES	0x04
#define UMASS_FLAGS_NO_AUTORUN 			0x08
#define UMASS_FLAGS_NO_UNKNOWN_DEVICES  0x10
#define UMASS_BOOST_UNKOWN_PRIORITY		0x20
#define UMASS_ENABLE_RESMGR_IFACE		0x40
#define UMASS_ENABLE_PROCESS_MONITOR	0x80
#define UMASS_ENABLE_IS_MTP_DEVICE     0x100 

extern ssize_t umass_slogf( int opcode, int severity, int verbosity, const char *fmt, ... );

/* prototypes for parsing functions */
int 				parser_get_config_data( const char *filename );
mass_device_info 	*parser_dev_entry( usbd_device_ident_t *dident, char *serial_string );
int 				parser_free_dev_entries( void );
current_device_info_t *find_cdevice_entry( _uint32 path, _uint32 devno );
int umass_change_mount( char *mount_point, _uint32 flags );
int umass_force_relearn( int busno, int devno );

#endif
